import 'dart:io';
import 'package:hamachi/src/hamachi_computer.dart';
import 'package:hamachi/src/hamachi_network.dart';
import 'package:hamachi/src/hamachi_status.dart';


class Hamachi{
  static final reNetwork = RegExp(r'^ [\* ] \[(?<id>\d{3}-\d{3}-\d{3})\](?<name>.*)capacity: (?<capacity>[^,]*), subscription type: (?<type>[^,]*), owner: (?<owner>.*)$');
  static final reComputer = RegExp(r'^[ ]{5}(?<status>[\* ]) (?<id>\d{3}-\d{3}-\d{3}) {3}(?<name>.{27})(?<ipv4>[\d\. ]{18})alias: (?<alias>.{18})(?:(?<ipv6>.{18})(?:(?:.{26})(?<type>.{12})(?<protocol>.{3})(?<proxy>.*)?)?)?');

  /// Get a List of computers in your hamachi networks.
  /// 
  static List<HamachiComputer> getList(){
    var list = <HamachiComputer>[];
    var lines = _getHamachiList().split('\n');
    // print(lines);
    HamachiNetwork network;
    lines.forEach((line) {
      var match = reNetwork.firstMatch(line);
      if (match!=null){ // first attempt to match a network
        network = HamachiNetwork.fromMatch(match);
      } else { // if not network it must be a computer
        match = reComputer.firstMatch(line);
        if (match!=null){
          try {
            var computer = HamachiComputer.fromMatch(match);
            computer.network = network;
            list.add(computer);
          } catch (error){
            print('Error parsing ${line}');
          }
        }
      }
    });
    // print(list);
    return list;
  }

  /// Get the connected status of a Hamachi client
  /// 
  static String getHamachiStatus(String hamId){
    String status;
    if (hamId.isNotEmpty){
      var peerString = _getHamachiPeer(hamId);
      var peer = HamachiStatus.fromList(peerString.split('\n'));
      // print(peer);
      if (peer!=null){
        status = peer.vpnStatus;
      }
    } else {
      // if this is not a hamachi computer, always return ok
      status = 'ok';
    }
    return status;
  }

  /// Get the details about a hamachi client
  /// 
  static HamachiComputer getHamachiComputer(String hamId){
    var hamComputer = getList().firstWhere((element) => element.id==hamId);
    return hamComputer;
  }

  /// Internal call to actually get the list
  /// 
  static String _getHamachiList(){
    var results = Process.runSync(
      '/bin/bash',
      ['-c','hamachi list'],
      runInShell:true,
      // mode:ProcessStartMode.normal
    );
    return results.stdout.toString();    
  }

  /// Internal call to get the status of a hamachi client
  /// 
  static String _getHamachiPeer(String hamId){
    return Process.runSync(
      '/bin/bash',
      ['-c','hamachi peer ${hamId}'],
      runInShell: true
    ).stdout.toString();
  }

}